breeds [
  initiators   ;; yellow
  monomers     ;; green
  polymers     ;; blue
  radicals     ;; red
]

turtles-own [
  chain-length
]

to setup
  ca
  set-default-shape turtles "circle"
  ;; make molecules
  create-custom-monomers 200
   [ set color green
     set chain-length 1 ]
  create-custom-initiators Io
   [ set color yellow
     set chain-length 0 ]
  ;; scatter them around the screen
  ask turtles
   [ setxy (random screen-size-x) (random screen-size-y) ]
  ;; start plots
  setup-plots
  do-plot
end

to go
  ask monomers [ grow-chain ]
  ask initiators [ initiate ]
  ask turtles [ move ]
  ask radicals [ terminate ]
  do-plot
  ifelse show-lengths?
    [ ask radicals [ set label chain-length ]
      ask polymers [ set label chain-length ] ]
    [ ask turtles [ set label no-label ] ]
end

to move
  rt random 50 - random 50
  fd 1
end

;; monomer procedure -- monomer turtle finds a radical on the same patch,
;; increases the chain-length of the radical by 1, then dies
to grow-chain
  locals [ partner ]
  set partner random-one-of radicals-here
  if (partner != nobody) and (random 1000 < Km)
    [ set chain-length-of partner (chain-length-of partner) + 1
      die ]
end

;; initiator procedure -- unimolecular initiation: yellow turtles
;; spontaneously break up into two red radicals
to initiate
  if (random 1000 < Ki)
    [ set color red
      set breed radicals
      hatch 1 [ move ] ]
end

;; radical procedure -- radicals react pairwise
to terminate
  locals [ partner ]
  set partner random-one-of other-radicals-here
  if (partner != nobody) and (random 500 < Km)
    [ if (who < who-of partner)
        [ set color-of partner blue
          set breed-of partner polymers
          set chain-length-of partner chain-length + chain-length-of partner
          die ] ]
end

;; plotting procedures

to setup-plots
  set-current-plot "Counts"
  set-plot-y-range 0 count turtles
end

to do-plot
  set-current-plot "Counts"
  set-current-plot-pen "initiator"
  plot count initiators
  set-current-plot-pen "monomer"
  plot count monomers
  set-current-plot-pen "radical"
  plot count radicals
  set-current-plot-pen "polymer"
  plot count polymers
  every 0.5 [ do-histogram ]
end

to do-histogram
  locals [ histogram-max ]
  set-current-plot "Chain Lengths"
  set histogram-max (max se 10 values-from polymers [chain-length])
  set-plot-x-range 0 histogram-max
  set-histogram-num-bars (histogram-max + 1)
  histogram polymers [chain-length]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Radical Polymerization model.
; http://ccl.northwestern.edu/netlogo/models/RadicalPolymerization.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
287
10
700
444
15
15
13.0
1
10
0
0

CC-WINDOW
287
445
742
563
Command Center

BUTTON
1
41
72
74
setup
setup
NIL
1
T
OBSERVER

BUTTON
73
41
152
74
go
go
T
1
T
OBSERVER

SLIDER
1
108
152
141
Ki
Ki
0.0
100.0
10.0
1.0
1
NIL

SLIDER
1
141
152
174
Km
Km
0.0
100.0
100.0
1.0
1
NIL

SLIDER
1
174
152
207
Kr
Kr
0.0
100.0
100.0
1.0
1
NIL

SLIDER
1
75
152
108
Io
Io
0
100
40
1
1
NIL

PLOT
1
207
279
369
Counts
time
#
0.0
50.0
0.0
240.0
true
true
PENS
"initiator" 1.0 0 -4013568 true
"monomer" 1.0 0 -11352576 true
"radical" 1.0 0 -65536 true
"polymer" 1.0 0 -16776961 true

PLOT
1
369
279
525
Chain Lengths
Length
#
0.0
10.0
0.0
10.0
true
false
PENS
"num-chains" 1.0 1 -16776961 true

SWITCH
153
58
286
91
show-lengths?
show-lengths?
0
1
-1000

@#$#@#$#@
WHAT IS IT?
------------
This model explains kinetics of radical polymerization.  Radical polymerization can be thought of as the process of agglomeration of small molecules (monomers) into chains (polymer chains) initiated by active particles (radicals).

An example of radical polymerization would be industrial polymerization of ethylene into polyethylene (PET):

|        n C H  --------> ~C  H~
|          2 4           2n 4n

A gaseous mixture of ethylene and oxygen (used as a catalyst), is pumped through a mile-long heated pipe.  As the mixture moves along the pipe, the molecules of ethylene polymerize into chains.  At the end of the pipe, the polymer is collected and co-products with low molecular mass are recycled.


Molecules of oxygen turn into bi-radicals under the influence of high temperature:

|        O --------> *O-O*
|         2

here the asterisk denotes a free electron.  These bi-radicals react with pi-bonds in ethylene and start the process of chain growth according to the scheme:


|        *O-O* + C H --------> *O-O-CH-CH*
|                 2 4                 2  2


|        *O-O-CH-CH* + CH-CH --------> *O-O-CH-CH-CH-CH*
|               2  2     2  2                 2  2  2  2


We can thus think of radical polymerization as interaction between (bi-) radicals and monomers.

Notice that if two radicals react with each other, they will form an inert compound and end the process of forming loner chains.

Let us assume that we run polymerization using AIBN as an initiator which we denote by I.  We thus have the following kinetically distinct particles in the system:

  (i) Initiator molecules;
  (ii) Radicals;
  (iii) Monomers;
  (iv) "Dead" chains;

A radical can react with a monomer, in which case it absorbs the monomer and increases its chain-length by 1.  Alternatively, a radical can react with another radical forming a "dead" molecule which does not react with other particles.  Let us denote by R(n) radicals which have reacted n times with monomers.  Let M stand for monomers and C(k) for "dead" chains of length k.  Then schematically we can describe the reactions by the following system of equations:

|        I --------> 2 R(1)              (initiation)
|
|        R(n) + M --------> R(n+1)       (growth)
|
|     R(n) + R(m) --------> C(n+m)       (termination)

In this model, there are four breeds of turtles:
- initiators (yellow turtles);
- monomers (green turtles);
- radicals (red turtles);
- polymers (blue turtles).

Each turtle carries a variable, chain-length.  Initially there are 200 turtles of monomer breed with chain-length set to 1.  There is also a user-defined number (Io) of initiator breed with chain-length set to 0.

The rules are quite simple:

(i) All turtles move around randomly.
(ii) Yellow turtles (initiators) spontaneously break up into two red turtles (radicals) with probability Ki;
(iii) Green turtles (monomers) react with red turtles (radicals) with probability Km.  If there is at least one radical on the same patch as the given monomer, the monomer dies and the chain-length of the radical which "killed" it is incremented by 1;
(iv) Red turtles (radicals) react with each other pairwise.  If there is a pair of radicals on the same patch, they annihilate each other with probability Kr forming a polymer chain (a blue turtle);


HOW TO USE IT
-------------
With sliders one can control the initial number of initiators (Io) and reaction probabilities
- Ki (initiator): probability of an initiator (yellow) creating two radicals (red);
- Km (monomer): probability of reaction between momomers (green) and radicals (red), forming a radical with a longer chain length;
- Kr (radical): probability of two radicals (red)  reacting to form a polymer chain (blue).

Select the initial number of initiators Io and press the "setup" button.  To start the simulation press the GO button.  You can set the probabilities Ki, Km and Kr before or after you do setup.  You can also modify this constants on the fly during the execution of the GO procedure.

One plot shows the number of initiators, monomers, radicals, and polymers.  The distribution of chain-lengths of polymers (blue) is shown in the second plot in the form of a histogram. If the SHOW-LENGTHS? switch is on, each chain will be labeled with its length.


THINGS TO NOTICE
----------------
Distribution of chain-length drastically depends on the mode of initiation.  If you select low values of Io and Ki, then the life expectancy of radicals will rise leading to longer chains.  However in this case the simulation will have to run too long in order to produce a sizable amount of long-length chains.   If you start the simulation with a lot of initiator which decomposes rapidly, radicals will tend to react with themselves and form low molecular oligomers.


THINGS TO TRY
-------------
Try to find the optimum values for which the distribution is tight and the simulation runs at a reasonable pace.


EXTENDING THE MODEL
-------------
Try to implement the model of industrial production of polyethylene (PET) described above.  Are you going to have "dead" chains in this model?

Can you modify the model so that visible "chains" of turtles actually form, like in the Follower model?


REFERENCES AND RELATED MODELS
-----------------------------
See also the series of "Simple Kinetics" models.

Note that in real polymerization reactions, the outcomes are affected by the position and shapes of the chains in three-dimensional space.  This aspect of the reactions is not presented in this model.

Although oxygen is widely used in industry as initiator of radical polymerization, scientists prefer to use other initiators in the lab, especially when the reaction has to be performed under conidions of atmospheric pressure and room temperature.  Two of the most frequently used lab initiators are benzoyl peroxide and 2-2'-azo-bis-isobutyrylnitrile (AIBN):

|        C H-C(O)O-O(O)C-H C
|     6 5             5 6
|
|            CH     CH
|        | 3    | 3
|        CH -C--N=N-C- CH
|      3 |      |    3
|        CN     CN


Peroxydes and azo-compounds are extremely unstable and tend to break up into radicals very easily if exposed to moderate heat or light.  Here is the reaction of decomposition of AIBN:

|            CH     CH                     CH
|        | 3    | 3                    | 3     ^
|        CH -C--N=N-C- CH  --------> 2 CH -C* + N  |
|      3 |      |    3               3 |     2
|        CN     CN                     CN


This reaction is irreversible because of the escape of nitrogen.

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Radical Polymerization model. http://ccl.northwestern.edu/netlogo/models/RadicalPolymerization. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

circle
true
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

@#$#@#$#@
NetLogo 1.3beta1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
