patches-own [
  firing?           ;; white cells
  refractory?       ;; red cells
  firing-neighbors  ;; counts how many neighboring cells are firing
]

to setup-blank
  ask patches
    [ cell-death ]
end

to setup-random
  ask patches
    [ ifelse random 100 < initial-density * 100
        [ cell-birth ]
        [ cell-death ] ]
end

to cell-birth  ;; patch procedure
  set firing? true
  set refractory? false
  set pcolor white
end

to cell-aging  ;; patch procedure
  set firing? false
  set refractory? true
  set pcolor red
end

to cell-death  ;; patch procedure
  set firing? false
  set refractory? false
  set pcolor black
end

to go
  if mouse-down?
    [ stop ]  ;; wait for user to stop drawing
  ask patches
    [ set firing-neighbors count neighbors with [firing?] ]
  ;; Starting a new "ask patches" here ensures that all the patches
  ;; finish executing the first ask before any of them start executing
  ;; the second ask.  This keeps all the patches in sync with each other,
  ;; so the births and deaths at each generation all happen in lockstep.
  ask patches
    [ ifelse firing?
        [ cell-aging ]
        [ ifelse refractory?
            [ cell-death ]
            [ if firing-neighbors = 2
                [ cell-birth ] ] ] ]
end

to add-cells [white?]
  if mouse-down?
    [ ask patch-at mouse-xcor mouse-ycor
        [ ifelse white?
            [ cell-birth ]
            [ cell-aging ] ] ]
end

to remove-cells
  if mouse-down?
    [ ask patch-at mouse-xcor mouse-ycor
        [ cell-death ] ]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Brian's Brain model.
; http://ccl.northwestern.edu/netlogo/models/BriansBrain.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
290
10
596
460
25
37
6.0
1
10
0
0

CC-WINDOW
4
319
286
460
Command Center

SLIDER
126
45
282
78
initial-density
initial-density
0.0
1.0
0.3
0.01
1
NIL

BUTTON
17
46
119
79
NIL
setup-random
NIL
1
T
OBSERVER

BUTTON
37
104
115
137
go-once
go
NIL
1
T
OBSERVER

BUTTON
126
104
217
137
go-forever
go
T
1
T
OBSERVER

BUTTON
17
10
119
43
NIL
setup-blank
NIL
1
T
OBSERVER

TEXTBOX
152
178
280
277
When one of these buttons is down, you can add or remove cells by holding down the mouse button and "drawing".

BUTTON
17
163
140
196
add white cells
add-cells true
T
1
T
OBSERVER

BUTTON
17
233
140
266
NIL
remove-cells
T
1
T
OBSERVER

BUTTON
17
198
140
231
add red cells
add-cells false
T
1
T
OBSERVER

@#$#@#$#@
WHAT IS IT?
-----------
This program is an example of a two-dimensional cellular automaton.  If you are not already familiar with 2D CA, see the model "Life" for a basic discussion.

Typical CAs use two cell states (live and dead), but Brian's Brian uses three: firing (white), refractory (red), and dead (black).

This CA is especially interesting to watch because it has many configurations that move steadily across the grid (as opposed to Life, which has only relatively few such configurations).


HOW IT WORKS
------------
Firing (white) cells always become refractory (red) at the next time step.

Refractory (red) cells always die (turn black) at the next time step.

A new firing (white) cell is born in any black cell that has exactly two firing (white) neighbors (of its eight surrounding cells).


HOW TO USE IT
-------------
The INITIAL-DENSITY slider determines the initial density of cells that are firing.  SETUP-RANDOM places these cells.  GO-FOREVER runs the rule forever.  GO-ONCE runs the rule once.

If you want to draw an initial pattern yourself, or add or remove cells in the middle of a run, turn on the ADD WHITE CELLS, ADD RED CELLS, and REMOVE CELLS buttons, then "draw" with the mouse in the graphics window.  


THINGS TO NOTICE
----------------
Lots of patterns stay stable and move steadily across the grid.  Such patterns are often referred to as "gliders".  How many different types of gliders do you see?  Why does this happen?  How do the rules of the CA result in this behavior?


THINGS TO TRY
-------------
Are there any stable shapes that don't move?

Are there any "glider guns" (objects that emit a steady stream of gliders)?

On a small enough grid, usually the CA reaches a steady state where there may be movement but nothing new happens.  In Brian's Brain, a square grid usually reaches a steady state more quickly than a rectangular grid (try it!).  Why?


EXTENDING THE MODEL
-------------------
Many other interesting 3-state 2D automata exist.  Experiment with variations on the rules in this model.


RELATED MODELS
--------------
See all of the other models in the "Cellular Automata" subsection of the "Computer Science" section of the NetLogo Models Library.


REFERENCES AND CREDITS
----------------------
Brian's Brain was invented by Brian Silverman.

To refer to this NetLogo model in academic publications, please use: Wilensky, U. (2002).  NetLogo Brian's Brain model.  http://ccl.northwestern.edu/netlogo/models/BriansBrain.  Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

@#$#@#$#@
NetLogo 1.2pre7
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
